// ======================================================================
//
// Gl_dd.cpp
// jeff grills
//
// copyright 1997 origin systems inc
//
// ======================================================================

#include "first.h"
#include "Gl_dd.h"

#include "ConfigPlatform.h"
#include "Gl_init.h"
#include "DirectDraw.h"
#include "PixelIndex.h"
#include "Palette.h"
#include "uvstruct.h"
#include "wind.h"
#include "WriteBMP.h"

// ======================================================================

void *GL_dd::buffer;
int   GL_dd::pitch;

// ======================================================================

GL_dd::GL_dd(void)
{
	buffer = NULL;
	pitch  = -1;
}

// ----------------------------------------------------------------------

GL_dd::~GL_dd(void)
{
	DirectDraw::remove();
	Wind::remove();
}

// ----------------------------------------------------------------------

bool GL_dd::init(const GL_input *input, GL_output *output, const DirectDraw::Driver *driver)
{
	if (GL_abstract::init(input, output))
	{
		// if no driver was specified, pick one now
		if (driver == NULL)
		{
			DirectDraw::enumerate();
			driver = DirectDraw::getDriver(ConfigPlatform::directDrawDevice());
		}

		if (!input->mfc)
			Wind::install(input->window, ConfigPlatform::windowed(), input->width, input->height);

		DirectDraw::install(input->window, driver, input->mfc);

		return true;
	}
	else
		return false;
}

// ----------------------------------------------------------------------

void GL_dd::setDrawWindow(HWND hwnd)
{
	DirectDraw::setDrawWindow(hwnd);
}

// ----------------------------------------------------------------------

bool GL_dd::gdiVisible(void)
{
	return DirectDraw::isGdiVisible();
}

// ----------------------------------------------------------------------

void GL_dd::createSurfaces(const GL_input *input, GL_output *output)
{
	DirectDraw::createSurfaces(input->width, input->height, input->depth * 8);

	// store the data that GL will need
	DEBUG_FATAL(input->depth < 2, ("Gl_dd::createSurfaces palettized"));

	// all DirectDraw GLs will default their movie destination settings to their frame buffer settings
	// each GL should override this in their ::init() after the ::createSurfaces() call, if necessary
	output->rMovieSizeShift   = DirectDraw::getRSizeShift();
	output->gMovieSizeShift   = DirectDraw::getGSizeShift();
	output->bMovieSizeShift   = DirectDraw::getBSizeShift();
	output->rMoviePlaceShift  = DirectDraw::getRPlaceShift();
	output->gMoviePlaceShift  = DirectDraw::getGPlaceShift();
	output->bMoviePlaceShift  = DirectDraw::getBPlaceShift();

	output->warningOneMegVideoCard = DirectDraw::isOneMegVideoCard();
	output->warningBackBufferInSystemRam = DirectDraw::isBackBufferInSystemRam();
	output->warningBankSwitched = DirectDraw::isBankSwitched();
}

// ----------------------------------------------------------------------

void GL_dd::unlock(bool display)
{
	if (display)
	{
		// snap a screen shot as necessary
		if (screenShotName && lockDD(false))
		{
			int      x, y;
			RGB     *rgb, *p;
			u_int32  color;

			// create storage for a 24bit version of the frame buffer
			rgb = NEW RGB[width * height];

			switch (depth)
			{
				case 2:
					// copy the pixels and promote to 24bit
					for (y = 0, p = rgb; y < height; y++)
						for (x = 0; x < width; x++, p++)
						{
							color = *pixelIndex(reinterpret_cast<Pixel16 *>(buffer), x, y, pitch);
							p->r = static_cast<u_int8>(((color >> DirectDraw::getRPlaceShift()) & (255 >> DirectDraw::getRSizeShift())) << DirectDraw::getRSizeShift());
							p->g = static_cast<u_int8>(((color >> DirectDraw::getGPlaceShift()) & (255 >> DirectDraw::getGSizeShift())) << DirectDraw::getGSizeShift());
							p->b = static_cast<u_int8>(((color >> DirectDraw::getBPlaceShift()) & (255 >> DirectDraw::getBSizeShift())) << DirectDraw::getBSizeShift());
						}
					break;

				default:
					DEBUG_FATAL(true, ("Gl_dd::unlock screenshot %d", depth));
					memset(rgb, 0, sizeof(rgb) * width * height);
					break;
			}

			// create the file
			writeBMP(screenShotName, rgb, width, height);

			delete [] rgb;

			// whack the output file name to avoid taking another screen shot
			delete [] screenShotName;
			screenShotName = NULL;
		}

		unlockDD();
		DirectDraw::slam();
	}
}

// ----------------------------------------------------------------------

bool GL_dd::lockDD(bool backBuffer)
{
	// unlock the previous lock because we do not know if it was a front or back buffer lock
	unlockDD();

	if (DirectDraw::screenLock(backBuffer))
	{
		buffer = DirectDraw::screenPtr();
		pitch  = DirectDraw::screenPitch();
		return true;
	}

	return false;
}

// ----------------------------------------------------------------------

void GL_dd::unlockDD(void)
{
	if (buffer)
	{
		DirectDraw::screenUnlock();
		buffer = NULL;
		pitch  = -1;
	}
}

// ----------------------------------------------------------------------

void GL_dd::movie(const uv_instance *uv)
{
	int mw = uv->width;
	int mh = uv->height;
	int dest_stride;
	int x,y;
	int *src, *dest;
	
	// make sure we have access to the LFB
	lockDD();

	src = reinterpret_cast<int *>(uv->last_buffer);

	if (uv->options.option_flags & X_CENTRE)
		if (uv->options.option_flags & (X_DOUBLE | X_SKIP))
			x = (width - (mw * 2)) / 2;
		else
			x = (width - mw) / 2;
	else
		if (uv->options.option_flags & X_OFFSET)
			x = uv->options.x_offset;
		else
			x = 0;

	if (uv->options.option_flags & Y_CENTRE)
		if (uv->options.option_flags & (Y_DOUBLE | Y_SKIP))
			y = (height - (mh * 2)) / 2;
		else
			y = (height - mh) / 2;
	else
		if (uv->options.option_flags & Y_OFFSET)
			y = uv->options.y_offset;
		else
			y = 0;

	dest = reinterpret_cast<int *>(pixelIndex(static_cast<Pixel16 *>(buffer), x, y, pitch));
	(int &)dest &= ~3;

	if ((uv->options.option_flags & X_DOUBLE) == 0)
		mw /= 2;
	
	dest_stride = pitch - (mw * sizeof(int));

	if (uv->options.option_flags & Y_SKIP)
		dest_stride += pitch;

	for (y = mh; y ; y--)
	{   
		for (x = mw; x; x--, src++, dest++)
			*dest = *src;

		if (uv->options.option_flags & Y_DOUBLE)
		{
			src -= mw;
			(int &)dest += dest_stride;

			for (x = mw; x; x--, src++, dest++)
				*dest = *src;
		}

		(int &)dest += dest_stride;
	}
}

// ----------------------------------------------------------------------

bool GL_dd::specialEffect(int id, void *data)
{
	return GL_abstract::specialEffect(id, data);
}

// ======================================================================


